import sys
import os
from optparse import OptionParser, SUPPRESS_HELP

import net
import random
import time
import instance

'''
import resource
rsrc = resource.RLIMIT_DATA
soft, hard = resource.getrlimit(rsrc)
print 'Soft limit starts as  :', soft
resource.setrlimit(rsrc, (307200000, hard)) #limit to one kilobyte
soft, hard = resource.getrlimit(rsrc)
print 'Soft limit changed to :', soft
'''

def main():
	(options, args) = parser.parse_args(sys.argv)
	print "OPTIONS"
	print options
	print "--"	
	
	inst=None
	try:
		start_time = time.time()
		inst=instance.instance(options)
		#graph=net.ResonanceGraph(options, inst)
		#graph.print_graph(inst, onlynodes=["20_21","16_19","14_9","9_10"])
		#exit()
		#print inst
		print("INSTANCE CREATION TIME: %.2f seconds" % (time.time() - start_time))
	except instance.NoInstance:
		print "No instance defined"
		exit()
	except instance.EmptyMu:
		print "Configuration vector mu is empty"
		exit()
	except instance.EmptyNu:
		print "Configuration vector nu is empty"
		exit()
	except instance.FormatErrorMuNu:
		print "Recheck either vector nu or mu, their size is different, it is impossible to compute k"
		exit()
	#print "IDEAL SPAN:",inst.get_ideal_span()
	
	if options.seed>0:
		random.seed(options.seed)

	if options.print_inst:
		print inst
		print
		if options.show_inst:
			inst.print_graph(spread=options.show_spread)
		print "EXITING..."
		exit()

	import maxpk_milpmodel as milp
	print "THIS PROGRAM MAXIMIZES THE BOTTOM PARALLELISM WITH MILP/GUROBI"
	graph=net.ResonanceGraph(options, inst, edges=False)
	if options.g:
		graph.print_graph(inst)
		exit()

	maxpar=inst.rowlen(-options.milp_k)
	while True:
		model=milp.ResonanceModel(graph, inst, options, maxpk=maxpar)

		stat=model.solve()
		#f.write(str(options.milp_d)+"\t")

		if stat=="inf":
			print "NO SOLUTION with k=", options.milp_k
			print "ENDING ITERATIVE PROCEDURE"
			#f.write("inf\t"+str(model.model.Runtime)+"\t-\t\n")
			#f.close()
			exit()
		if stat=="op":
			maxpar=model.get_solbottompar()
			print "max:P("+str(options.milp_k)+")="+str(maxpar)
			model.print_sol()
		if not options.milp_it:
			print "ITERATION ENDED... EXITING"
			exit()


parser = OptionParser()
#GENERAL OPTIONS
parser.add_option("--debug", dest="debug", action="store_true", default=False, help="enable debug")
parser.add_option("--seed", dest="seed",  default=0, type="int", help="seed")
parser.add_option("--print-inst", dest="print_inst", action="store_true", default=False, help="Print instance and exit")
parser.add_option("--print-asp", dest="print_asp", action="store_true", default=False, help="Print ASP instance and exit")
parser.add_option("--show-inst", dest="show_inst", action="store_true", default=False, help="Show instance and exit (requires --print-inst)")
parser.add_option("--read-inst", dest="filename", default=None, metavar="FILE", help="Show instance and exit (requires --print-inst)")
parser.add_option("--show-spread", dest="show_spread", action="store_true", default=False, help="Spread the resonances in the graph")

#TABLE CREATION PARAMETERS
parser.add_option("--Lmin", dest="lambdamin", type="float", default=1491.0, help="Minimum value of lambda")
parser.add_option("--Lmax", dest="lambdamax", type="float", default=1611.0, help="Maximum value of lambda")
parser.add_option("--Rmin", dest="rmin", type="float", default=5.0, help="Rmin")
parser.add_option("--Rmax", dest="rmax", type="float", default=25.0, help="Rmax")
parser.add_option("--Rstep", dest="rstep", type="float", default=1.0, help="Step radius")
parser.add_option("--Rtol", dest="rtol", type="float", default=0.0, help="Fabrication uncertainty for radius values")
parser.add_option("--laser-prec", dest="laserprec", type="int", default=0, help="{0,1,2} If 0 the laser has nm precision, if 1 has nm.x precision, if 2 has nm.xy precision.")
parser.add_option("--DeltaL", dest="design_deltalambda", type="float", default=0.0, help="uniform delta-lambda, laser uncertainty")
#parser.add_option("--mfree", dest="mfree", type="int", default=0, help="Create m 'free' radiuses (a channel without PSEs)")
#parser.add_option("--mfree-step", dest="mfreestep", type="float", default=0.5, help="additional step to create the resonance of the 'free' radiuses")

parser.add_option("--milp-thr", dest="milp_threads", type="int", default=1, help="MILP MODE: Number of threads")
parser.add_option("--milp-it", dest="milp_it", action="store_true", default=False, help="MILP MODE: iterative mode")
parser.add_option("--milp-k", dest="milp_k", type="int", default=1, help="MILP MODE: Number of radius values to be selected")
parser.add_option("--milp-p", dest="milp_p", type="int", default=1, help=SUPPRESS_HELP)
parser.add_option("--milp-d", dest="milp_d", type="float", default=0, help=SUPPRESS_HELP)
parser.add_option("--milp-of", dest="milp_of", type="float", default=999999, help=SUPPRESS_HELP)
parser.add_option("--milp-to", dest="milp_to", type="float", default=999999, help="MILP MODE: time_limit")
parser.add_option("--milp-graph-in", dest="g", action="store_true", default=False, help="MILP MODE: create the instance's graph and exit")
parser.add_option("--milp-graph-sol", dest="s", action="store_true", default=False, help="MILP MODE: create the solution's graph")
parser.add_option("--milp-cns-ma", dest="maxarea", default=-1, type="int", help="MILP MODE: Maximum area")
parser.add_option("--milp-uncns-me", dest="milp_uncns_me", action="store_false", default=True, help="MILP MODE: disable radius incompatibility cns")
parser.add_option("--milp-uncns-up", dest="milp_uncns_up", action="store_false", default=True, help="MILP MODE: disable uniform parallelism (cardinality) cns")
parser.add_option("--milp-uncns-rc", dest="milp_uncns_rc", action="store_false", default=True, help="MILP MODE: disable radius cardinality cns")
parser.add_option("--milp-uncns-pc", dest="milp_uncns_pc", action="store_false", default=True, help="MILP MODE: disable path cardinality cns")

if __name__ == "__main__":
    main()
