import networkx as nx
#import matplotlib.pyplot as plt

class ResonanceGraph(object):
	def __init__(self, options, instance, edges=True):
		self.enable_edges=edges
		self.nrows=instance.nrows()
		self.milp_d=options.milp_d
		self.ideal_span=instance.get_ideal_span(options.milp_p, options.milp_k)
		self.nodeval={}
		self.createNet(options, instance)
		self.print_stats()
		#self.print_graph()
		#self.check_connectivity()

	def is_admitted(self, diff, delta):
		if delta==-1:
			return True
		else:
			return diff>=self.ideal_span-delta and diff<=self.ideal_span+delta

	def createNet(self, options, instance):
		print "CREATING GRAPH..."
		self.net=nx.DiGraph()
		for row in range(instance.nrows()):
				for el in range(instance.rows()[row]['len']):
					self.net.add_node(str(row)+"_"+str(el))
					self.nodeval[self._nodename(row,el)]=instance.row(row)['res'].getval(el)

		#S,T creation...
		self.net.add_node("S")
		self.net.add_node("T")	

		#if maximization of bottom parallelism the edges are not necessary
		if not self.enable_edges:
			return
		#the net can map the constraints about vertical and horizontal tolerances: the edges are not included into the graph
		for rowa in range(instance.nrows()):
			for rowb in range(instance.nrows()):
				#if abs(rowa-rowb)<self.instance.vetol: #todo must be modified with rays and tols
				#	continue
				if rowa==rowb:
					if options.milp_p==1:
						continue
				for ela in range(instance.row(rowa)['len']):
					for elb in range(instance.row(rowb)['len']):
						#if abs(ela-elb)<self.instance.hotol and ela!=elab: #todo must be modified with sigmas and resvals
						#	continue
						resa=instance.row(rowa)['res'].getres(ela)
						resb=instance.row(rowb)['res'].getres(elb)
						vala=instance.row(rowa)['res'].getval(ela)
						valb=instance.row(rowb)['res'].getval(elb)
						#print abs(vala-valb),self.instance.is_admitted(abs(vala-valb))
						if vala<valb and self.is_admitted(abs(vala-valb), options.milp_d):# and not resa.interferes(resb.val,resb.sigma):
							#print "xxx",rowa+1,ela+1,rowb+1,elb+1,abs(self.instance.get_ideal_span()-abs(vala-valb))
							self.net.add_weighted_edges_from([(self._nodename(rowa,ela),self._nodename(rowb,elb),\
																abs(self.ideal_span-abs(vala-valb)))])

		#linking nodes to S and T
		Sedges=[]
		Tedges=[]
		for n in self.net.nodes():
			if n not in ["T","S"]:# and self.net.in_degree(n) == 0:
				Sedges.append(("S",n,0))
			if n not in ["T","S"]:# and self.net.out_degree(n) == 0:
				Tedges.append((n,"T",0))
		self.net.add_weighted_edges_from(Sedges)
		self.net.add_weighted_edges_from(Tedges)
		
		print "DONE!"

	def edges(self):
		return self.net.edges()

	def nodes(self):
		return self.net.nodes()

	def in_edges(self,node):
		return self.net.in_edges(node)

	def out_edges(self,node):
		return self.net.out_edges(node)

	def get_row(self, node):
		return node.split("_")[0]

	def get_col(self, node):
		return node.split("_")[1]

	def check_connectivity(self):
		flowG=self.net.copy()
		for e in flowG.edges():
			ee=flowG.edge[e[0]][e[1]]
			ee['weight']=1

		flow=nx.maximum_flow_value(flowG,"S","T", capacity='weight')
		print "FLOW VALUE IS",flow
		if flow==0:
			print "THE GRAPH IS NOT CONNECTED. TRY TO ENLARGE DELTA"
		else:
			print "CONGRATS! THE GRAPH IS CONNECTED"
	
	def _nodename(self,rowa,ela):
		return str(rowa)+"_"+str(ela)

	def __str__(self):
		print ""

	def print_stats(self):
		print "GRAPH HAS"
		g=self.net.to_undirected()
		print "IDEAL SPAN:",self.ideal_span
		print len(g.nodes()),"NODES"
		print "TAU:",self.milp_d
		print len(g.edges()),"EDGES AMONG NODES (EXCLUDING T/S)"

		g.remove_node("T")
		g.remove_node("S")
		print nx.number_connected_components(g),"CONNECTED COMPONENTS"

	def print_graph(self, instance, onlyedges=None, onlynodes=None):
		A=nx.to_agraph(self.net)				
		
		if onlyedges!=None:
			nodes=A.nodes()
			A.remove_edges_from(A.edges())
			A.add_nodes_from(nodes)
			oe=[]
			for e in onlyedges:
				if e[0]=="S" or e[1]=="T":
					continue
				A.add_edge(e[0],e[1],color='green',headlabel=str(abs(self.nodeval[e[0]]-self.nodeval[e[1]])),fontsize=50,labelfontcolor='blue')
		
		A.remove_node("S")
		A.remove_node("T")

		for n in A.nodes():
			nn=A.get_node(n)
			if "_" in n:
				r,e=n.split("_")
				r=int(r)
				e=int(e)
				nn.attr['pos']="%f,%f"%(float(e*200),float(self.nrows*200-r*200))
				nn.attr['fontsize']="50.0"
				nn.attr['shape']="none"
				if len(A.neighbors(nn))==0:
					nn.attr['color']="red"

				if onlynodes!=None and n in onlynodes:
					nn.attr['shape']="box"
					nn.attr['style']="rounded,filled,bold"
					nn.attr['fillcolor']="green"
				nn.attr['label']=self.nodeval[n]
			else:
				if n=="S":
					nn.attr['pos']="%f,%f"%(-500,float(self.nrows)*200/2)
					nn.attr['fontsize']="50.0"
					nn.attr['shape']="none"
				if n=="T":
					nn.attr['pos']="%f,%f"%(instance.sizebiggest*500+500,float(self.nrows)*200/2)
					nn.attr['fontsize']="50.0"
					nn.attr['shape']="none"

		sol=""
		if onlyedges!=None:
			sol="_sol"
		A.write(instance.name+sol+".dot")
		A.draw(instance.name+sol+".png",prog='neato',args='-n2')



'''
	#se calcolo il BFS ottengo tutti i cammini da S a T (polinomiali?? dato che il grafo e' aciclico)
	#quindi escludo tutti i cammini che hanno len!=K+2
	#in realta' essendo il numero di path polinomiali posso plottare il pareto front esatto...
	def brute_force(self):
		print "a"
		pathgen=nx.all_simple_paths(self.net, source='S', target='T')#, cutoff=self.K+1)
		print "b"
		paths=list(pathgen)
		print len(paths), "paths with length",self.K+1,"for DAG with", len(self.net.nodes()), "nodes and", len(self.net.edges()),"edges"
		#for p in paths:
		#	print paths
'''
